==> cryptide-1.3.1.tgz: package/src/tfastAuthFlow.js <==
// Tide Protocol - Infrastructure for the Personal Data economy
// Copyright (C) 2019 Tide Foundation Ltd
// 
// This program is free software and is subject to the terms of 
// the Tide Community Open Source License as published by the 
// Tide Foundation Limited. You may modify it and redistribute 
// it in accordance with and subject to the terms of that License.
// This program is distributed WITHOUT WARRANTY of any kind, 
// including without any implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE.
// See the Tide Community Open Source License for more details.
// You should have received a copy of the Tide Community Open 
// Source License along with this program.
// If not, see https://tide.org/licenses_tcosl-1-0-en

import EcScalar from './ecScalar';
import TSignClient, { TSignGetClient } from './tsignClient';
import ecSecretShare from './ecSecretShare';
import Hash from './hash';
import BigInt from 'big-integer';
import ElGamalKey from './elGamalKey';
import ElGamal from './elGamal';

export default class TFastAuthFlow {
    
    /**
     * @param {string[]} urls
     * @param {string} user
     */
    constructor(urls, user, isPost = false) {
        /**
        * @type {TSignClient[]}
        */
        this.clients = urls.map(url => new (isPost ? TSignClient : TSignGetClient)(url, user));
        this.user = user;
    }

    /**
     * @param {EcScalar} key
     * @param {string} pass
     * @param {number} threshold
     */
    async signIn(key, pass, threshold) {
        try {
            var m = getM(pass);
            var ids = (await Promise.all(this.clients.map(cln => cln.getId()))).map(id => new EcScalar(id));
            var passShrs = await getShares(m, ids, threshold);
            var keyShrs = await getShares(key, ids, threshold);
            await Promise.all(this.clients.map((cln, i) => cln.SetAccount(passShrs[i], keyShrs[i])));
        } catch (err) {
            return Promise.reject(err);
        }
    }

    /**
     * @param {string} password
     * @param {number} threshold
     */
    async logIn(password, threshold) {
        try {
            var ids = (await Promise.all(this.clients.map(cln => cln.getId()))).map(id => new EcScalar(id));
            var ais = await getShares(getM(password), ids, threshold);

            var cs = await Promise.all(this.clients.map((cln, i) => cln.getFastCi(ais[i], threshold, ids)));
            var shrCs = cs.reduce((a, b) => a.concat(b));
            var allCs = ids.map(cln => shrCs.filter(shr => shr.to.equals(cln)));
            for (var shrCi of cs) {
                var first = shrCi[0];
                var li = ecSecretShare.getLi(first.from, ids);
                var ciuili = first.ciui.times(li);
                for (const cj of shrCi) {
                    cj.ciui = ciuili;
                }
            }
            var rs = await Promise.all(this.clients.map((cln, i) => cln.getFastRj(allCs[i])));
            var shrRs =  rs.reduce((a, b) => a.concat(b));
            var allRs = ids.map(cln => shrRs.filter(shr => shr.to.equals(cln)));
            
            var [sec, pub] = ElGamalKey.generate(128);
            var keyTags = await Promise.all(this.clients.map((cln, i) => cln.getFastKey(allRs[i], pub)));

            var priv = keyTags.map(cypher => new EcScalar(ElGamal.decrypt(cypher.priv, sec))).reduce((sum, keyi) => sum.add(keyi), EcScalar.zero);
            var { p, q, g } = keyTags[0].pub;

            return { priv: new ElGamalKey(true, p, q, g, priv.value), pub: keyTags[0].pub };
        } catch (err) {
            return Promise.reject(err);
        }
    }
}

/**
 * @param {EcScalar[]} ids
 * @param {EcScalar} password
 * @param {number} threshold
 */
async function getShares(password, ids, threshold) {
    var [, shrs] = ecSecretShare.shareFromIds(password, ids.map(id => id), threshold);
    return shrs;
}

/**
 * @param {string} pass
 */
function getM(pass) {
    return new EcScalar(BigInt.fromArray(Array.from(Hash.shaBuffer(pass)), 256, false));
}

==> cryptide-1.5.1.tgz: package/src/tfastAuthFlow.js <==
// Tide Protocol - Infrastructure for the Personal Data economy
// Copyright (C) 2019 Tide Foundation Ltd
// 
// This program is free software and is subject to the terms of 
// the Tide Community Open Source License as published by the 
// Tide Foundation Limited. You may modify it and redistribute 
// it in accordance with and subject to the terms of that License.
// This program is distributed WITHOUT WARRANTY of any kind, 
// including without any implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE.
// See the Tide Community Open Source License for more details.
// You should have received a copy of the Tide Community Open 
// Source License along with this program.
// If not, see https://tide.org/licenses_tcosl-1-0-en

import EcScalar from './ecScalar';
import TSignClient, { TSignGetClient } from './tsignClient';
import ecSecretShare from './ecSecretShare';
import Hash from './hash';
import BigInt from 'big-integer';
import ElGamalKey from './elGamalKey';
import ElGamal from './elGamal';

export default class TFastAuthFlow {
    
    /**
     * @param {string[]} urls
     * @param {string} user
     */
    constructor(urls, user, isPost = false) {
        /**
        * @type {TSignClient[]}
        */
        this.clients = urls.map(url => new (isPost ? TSignClient : TSignGetClient)(url, user));
        this.user = user;
    }

    /**
     * @param {EcScalar} key
     * @param {string} pass
     * @param {number} threshold
     */
    async signIn(key, pass, threshold) {
        try {
            var m = getM(pass);
            var ids = (await Promise.all(this.clients.map(cln => cln.getId()))).map(id => new EcScalar(id));
            var passShrs = await getShares(m, ids, threshold);
            var keyShrs = await getShares(key, ids, threshold);
            await Promise.all(this.clients.map((cln, i) => cln.SetAccount(passShrs[i], keyShrs[i])));
        } catch (err) {
            return Promise.reject(err);
        }
    }

    /**
     * @param {string} password
     * @param {number} threshold
     */
    async logIn(password, threshold) {
        try {
            var keyPromise = ElGamalKey.generateAsync(128);
            var ids = (await Promise.all(this.clients.map(cln => cln.getId()))).map(id => new EcScalar(id));
            var ais = await getShares(getM(password), ids, threshold);

            var cs = await Promise.all(this.clients.map((cln, i) => cln.getFastCi(ais[i], threshold, ids)));
            var shrCs = cs.reduce((a, b) => a.concat(b));
            var allCs = ids.map(cln => shrCs.filter(shr => shr.to.equals(cln)));
            for (var shrCi of cs) {
                var first = shrCi[0];
                var li = ecSecretShare.getLi(first.from, ids);
                var ciuili = first.ciui.times(li);
                for (const cj of shrCi) {
                    cj.ciui = ciuili;
                }
            }
            var rs = await Promise.all(this.clients.map((cln, i) => cln.getFastRj(allCs[i])));
            var shrRs =  rs.reduce((a, b) => a.concat(b));
            var allRs = ids.map(cln => shrRs.filter(shr => shr.to.equals(cln)));
            
            var [sec, pub] = await keyPromise;
            var keyTags = await Promise.all(this.clients.map((cln, i) => cln.getFastKey(allRs[i], pub)));

            var priv = keyTags.map(cypher => new EcScalar(ElGamal.decrypt(cypher.priv, sec))).reduce((sum, keyi) => sum.add(keyi), EcScalar.zero);
            var { p, q, g } = keyTags[0].pub;

            return { priv: new ElGamalKey(true, p, q, g, priv.value), pub: keyTags[0].pub };
        } catch (err) {
            return Promise.reject(err);
        }
    }
}

/**
 * @param {EcScalar[]} ids
 * @param {EcScalar} password
 * @param {number} threshold
 */
async function getShares(password, ids, threshold) {
    var [, shrs] = ecSecretShare.shareFromIds(password, ids.map(id => id), threshold);
    return shrs;
}

/**
 * @param {string} pass
 */
function getM(pass) {
    return new EcScalar(BigInt.fromArray(Array.from(Hash.shaBuffer(pass)), 256, false));
}
